/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * MULTICLASSMODEL_interface.h
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MULTICLASSMODEL_INTERFACE_H_
#define _MULTICLASSMODEL_INTERFACE_H_
#include <armadillo>
#include <boost/serialization/vector.hpp>
#include "pcvmmodel.h"
#include "pcvmtrain_interface.h"
#include <iostream>
#include <boost/serialization/serialization.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/split_member.hpp>
#include <boost/serialization/shared_ptr.hpp> 
#include <boost/serialization/vector.hpp>
#include <boost/shared_array.hpp>
using namespace std;
using namespace arma;

class IMultiClassModelInterface
{
public:	
	virtual ~IMultiClassModelInterface() {};
	virtual void Init(const mat&) {};
	virtual mat  CalculateModelPredictions(const IKernelMatrixInterface &oKernelInterface) const = 0;
	virtual void GenerateMultiClassModels(IPCVMTrainInterface &oPCVMTrain,const IKernelMatrixInterface &oKernel, const mat &Data, const vec &Label) = 0;
	virtual const std::vector<std::vector<double> >& GetReferenceData(void) const = 0;
	virtual arma::vec GetUniqueLabels(void) = 0;
	virtual std::vector<pcvm_model> GetModels(void) = 0;
	virtual arma::uvec GetReferenceIndices(void) = 0;
private:
/*	template< class Archive >
	inline void serialize( Archive& ar, unsigned int const );
*/
	friend class boost::serialization::access;
	template< class Archive >
		inline void save( Archive& ar, const unsigned int ) const;
	template< class Archive >
		inline void load( Archive& ar, unsigned int const );
	BOOST_SERIALIZATION_SPLIT_MEMBER()
};

/*	template< class Archive >
	inline void IMultiClassModelInterface::serialize( Archive& ar, unsigned int const )
 	{};*/

template< class Archive >
		inline void IMultiClassModelInterface::save( Archive& ar, const unsigned int ) const
		{};

template< class Archive >
		inline void IMultiClassModelInterface::load( Archive& ar, unsigned int const )
		{std::cout << "mc model base" <<std::endl;};
#endif // _MULTICLASSMODEL_INTERFACE_H_

