/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * nystroem-kernel-matrix.h
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _NYSTROEM_KERNEL_MATRIX_H_
#define _NYSTROEM_KERNEL_MATRIX_H_
#include <armadillo>
#include "kernel-matrix.h"
using namespace arma;
class NystroemKernelMatrix: public KernelMatrix 
{
public:
	NystroemKernelMatrix();
	~NystroemKernelMatrix();
	// generate the kernel matrix from the given vectorial data
	// if needed optional parameters can be provided in Parameter
	bool Initialize(const mat &DataX, const mat &DataY, const mat &Parameter = zeros<mat>(1,1));
	mat  Evaluate(const uvec &X, const uvec &Y) const;
	void size(unsigned int &rows, unsigned int &cols) const;
	IKernelMatrixInterface* clone() const {
        return new NystroemKernelMatrix(*this);
    }
	// without selector 
	const mat& GetParameterC() const {return m_C;};
	const mat& GetParameterP() const {return m_Wpinv;};
	mat GetParameter (int iParameter) const;
protected:

private:
	bool GenerateKernelMatrix(const mat &DataX, const mat &DataY, const mat &Parameter);
	uvec m_vXSelector;
	uvec m_vYSelector;
	mat  m_C;
	mat  m_Wpinv;
	uvec m_Landmarks;
	bool m_status;
};

#endif // _NYSTROEM_KERNEL_MATRIX_H_

