/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * pcvmmodel.h
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _PCVMMODEL_H_
#define _PCVMMODEL_H_
#include <armadillo>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/split_free.hpp>
#include "kernel-matrix_interface.h"
// Forward declaration of class boost::serialization::access
using namespace arma;

//// Declaration of the template
class pcvm_model;

// Forward declaration of class boost::serialization::load
namespace boost {
namespace serialization {
	template<typename Archive> void save(Archive& ar, const pcvm_model& o, const unsigned int version);
	template<typename Archive> void load(Archive& ar, pcvm_model& o, const unsigned int version);
}
}

class pcvm_model
{
	public:
		pcvm_model(arma::vec Weights=arma::zeros<arma::vec>(1,1), double dBias=0.0, int iKernelType=0);
		// for correct usage of this predict method look into the provided examples
		// The kernel matrix is the full (multiclass model) with ReferenceIndices
		// indicating the row numbers effectively used in this specific model
		vec CalculatePrediction(const IKernelMatrixInterface &Kernel,arma::uvec ReferenceIndices) const;
		// predict with a already preprocessed (fitting) kernel
		vec CalculatePrediction(const mat &RawKernelMatrix) const;
		arma::vec GetWeights(void) const;
		arma::uvec GetSupportIndices(void) const;
		int GetKernelType(void) const;
		double GetBias(void) const;		
	protected:
		
	private:
		arma::vec Predict(arma::mat Data) const;
		arma::vec m_Weights;
		double    m_dBias;
		int       m_iKernelType;
		arma::uvec m_SupportIndices;
		// Allow serialization to access non-public data members.
		template<typename Archive> friend void boost::serialization::save(Archive& ar, const pcvm_model& o, const unsigned int version);
		template<typename Archive> friend void boost::serialization::load(Archive& ar, pcvm_model& o, const unsigned int version);
};
BOOST_SERIALIZATION_SPLIT_FREE(pcvm_model)
namespace boost {
namespace serialization {

		template<typename Archive>
		void save(Archive& ar, const pcvm_model& oClass, const unsigned version) {
			typedef std::vector<double> stdvec;
			typedef std::vector<unsigned int> uintvec;
			stdvec vDoubleVec;
			uintvec vVec2;
			arma::vec vTemp = oClass.GetWeights();
			arma::uvec vSupportIndices = oClass.GetSupportIndices();
			vDoubleVec = arma::conv_to< stdvec >::from(vTemp);
			vVec2 = arma::conv_to< uintvec >::from(vSupportIndices);
			//std::copy(vDoubleVec.begin(),vDoubleVec.end(),std::ostream_iterator<double>(std::cout," [pcvm-s] "));
			double dTemp = oClass.GetBias();
			int    iKernelType = oClass.GetKernelType();
			ar & dTemp & vDoubleVec & iKernelType & vVec2;  // Simply serialize the data members of Obj
		}

		template<class Archive>
			void load(Archive & ar, pcvm_model& oClass,  const unsigned int version) {
			ar & oClass.m_dBias; 
			std::vector<double> vDoubleVec;
			std::vector<unsigned int> 	vVec2;
			ar & vDoubleVec;
			//std::copy(vDoubleVec.begin(),vDoubleVec.end(),std::ostream_iterator<double>(std::cout," [pcvm-l] "));
			oClass.m_Weights = zeros<vec>(vDoubleVec.size());
			oClass.m_Weights = arma::conv_to< arma::vec >::from(vDoubleVec);
			ar & oClass.m_iKernelType; 
			ar & vVec2;
			oClass.m_SupportIndices = arma::conv_to< arma::uvec >::from(vVec2);									
		}	
	} // namespace serialization		
} // namespace boost


#endif // _PCVMMODEL_H_
