/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * pcvmmulticlassmodel_ecoc.h
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _PCVMMULTICLASSMODEL_ECOC_H_
#define _PCVMMULTICLASSMODEL_ECOC_H_
#include "multiclassmodel_interface.h"
#include <boost/serialization/serialization.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/split_member.hpp>
#include <boost/serialization/shared_ptr.hpp> 
#include <iterator>
using namespace arma;


// just a simple container object (lazy version)
class pcvm_multiclass_model_ecoc: public IMultiClassModelInterface
{
	public:
		pcvm_multiclass_model_ecoc() {}; // default constructor - use load to fill object or init
		void Init(const mat& CodingMatrix) {sm_CodingMatrix = CodingMatrix;};
		pcvm_multiclass_model_ecoc(const mat& CodingMatrix) {sm_CodingMatrix = CodingMatrix;};
		mat  CalculateModelPredictions(const IKernelMatrixInterface &oKernelInterface) const;
		void GenerateMultiClassModels(IPCVMTrainInterface &oPCVMTrain,const IKernelMatrixInterface &oKernel, const mat &Data, const vec &Label);
		const std::vector<std::vector<double> >& GetReferenceData(void) const {return sm_vvReferenceData; } ;
		arma::vec GetUniqueLabels(void)  {return sm_vUniqueLabel;} ;
		std::vector<pcvm_model> GetModels(void)  {return sm_Model; } ;
		arma::uvec GetReferenceIndices(void) {return sm_vReferenceIndices;};
		arma::mat GetCodingMatrix(void) {return sm_CodingMatrix; };
		std::vector<pcvm_model> sm_Model;
		std::vector<std::vector<double> > sm_vvReferenceData;
		std::vector<std::vector<unsigned int> > m_vvClassIndices;
		arma::uvec sm_vReferenceIndices;		
		arma::vec sm_vUniqueLabel;	
		arma::mat sm_CodingMatrix;
		template< typename t_Archive >
		inline void save( t_Archive& archive, unsigned int const ) const;

		template< typename t_Archive >
		inline void load( t_Archive& archive, unsigned int const );

		BOOST_SERIALIZATION_SPLIT_MEMBER()

	protected:

	private:
		pcvm_model CalculateClassificationModel (int i,IPCVMTrainInterface &oPCVMTrain,const IKernelMatrixInterface &oKernel, const vec &Label) const;
		// Allow serialization to access non-public data members.
		friend class boost::serialization::access;
};
BOOST_CLASS_VERSION(pcvm_multiclass_model_ecoc, 1)

template<typename OutType,typename InType> OutType mat_to_std_vec(const arma::mat &A) {
			OutType V(A.n_rows);
			for (size_t i = 0; i < A.n_rows; ++i) {
    			V[i] = arma::conv_to< InType >::from(A.row(i));
			};
			return V;
		}	

template<typename Archive>
		void pcvm_multiclass_model_ecoc::save(Archive& ar, const unsigned version) const {
			const pcvm_multiclass_model_ecoc& oClass = *this;
			ar & boost::serialization::base_object<IMultiClassModelInterface>(*this);
			typedef std::vector<double> stdvec;
			typedef std::vector<stdvec> stdvecvec;
			typedef std::vector<unsigned int> uintvec;
			stdvec vDoubleVec;		
			uintvec vVec2;
			stdvec  vCodingMatrix;
			vDoubleVec = conv_to< stdvec  >::from(oClass.sm_vUniqueLabel);
			vVec2      = conv_to< uintvec >::from(oClass.sm_vReferenceIndices);
			const double* vCodingEntries = oClass.sm_CodingMatrix.memptr();
			vCodingMatrix.reserve(oClass.sm_CodingMatrix.n_elem);
			std::copy(&vCodingEntries[0], &vCodingEntries[oClass.sm_CodingMatrix.n_elem], std::back_inserter(vCodingMatrix));
			//std::copy(vDoubleVec.begin(),vDoubleVec.end(),std::ostream_iterator<double>(std::cout," [multi-s] "));
			ar & vDoubleVec;  // Simply serialize the data members of Obj
			ar & vVec2;
			ar & oClass.sm_Model;
			ar & oClass.sm_vvReferenceData;
			ar & oClass.m_vvClassIndices;
			ar & vCodingMatrix;
		}

template<typename Archive>
			void pcvm_multiclass_model_ecoc::load(Archive & ar, const unsigned int version) { 
			std::cout << "ECOC"<< std::endl;
			pcvm_multiclass_model_ecoc& oClass = *this;
			ar & boost::serialization::base_object<IMultiClassModelInterface>(*this);
			std::vector<double> vDoubleVec;
			std::vector<unsigned int> vVec2;
			typedef std::vector<double> stdvec;
			stdvec vCodingMatrix;
			ar & vDoubleVec;
			ar & vVec2;
			//std::copy(vDoubleVec.begin(),vDoubleVec.end(),std::ostream_iterator<double>(std::cout," [multi-l] "));
			oClass.sm_vUniqueLabel = zeros<vec>(vDoubleVec.size());
			oClass.sm_vUniqueLabel = conv_to< arma::vec >::from(vDoubleVec);
			oClass.sm_vReferenceIndices = conv_to< arma::uvec >::from(vVec2);
			ar & oClass.sm_Model;
			ar & oClass.sm_vvReferenceData;
			ar & oClass.m_vvClassIndices;
			ar & vCodingMatrix;
			int C = oClass.sm_vUniqueLabel.n_elem;
			int N = vCodingMatrix.size();
			int L = ceil(N / C);
			oClass.sm_CodingMatrix = reshape(mat(vCodingMatrix),C,L);
				std::cout << oClass.sm_CodingMatrix << std::endl;
		}	


#endif // _PCVMMULTICLASSMODEL_ECOC_H_
