/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * SVD_NY_SYM_IMPL.hpp
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Implementation of an elm kernel in acc to
 * Frenay & Verleysen, Parameter Free kernel in extreme learnin ...,
 * Neurocomputing
 * Author: Frank-Michael Schleif
 */
#ifndef _SVD_NY_SYM_IMPL_HPP_
#define _SVD_NY_SYM_IMPL_HPP_
#include <iostream>
#include <armadillo>
#include <cmath>
#include "eig_ny.hpp"
#include "svd_ny_sym.hpp"
using namespace std;
using namespace arma;

/*
% Singular value decomposition based on a nystroem approximated -symmetric - matrix
% The left-singular vectors of K are eigenvectors of KK*.
% The right-singular vectors of K are eigenvectors of K*K.
% The non-zero singular values of K (found on the diagonal entries of sigma) are the square roots of the non-zero eigenvalues of both M*M and MM*.
%
% Note: 
%      1)   valid rank of U,V is obviously limited to the number of non-zero singular values
% This is correct upto the sign of the singular vectors
*/

void svd_ny_sym(const mat &C, const mat &Wpinv, const uvec &vLandmarks, mat &U, mat &S)
{
	unsigned int N   = C.n_rows;
	unsigned int m	 = C.n_cols;
	mat nyKKC,nyKKW;

	// % right singular vectors - V
	mat Temp = (C * Wpinv);
	Temp     = (Temp.t() * C);
	mat Ckk  = C*Temp;
	Temp     = Ckk.rows(vLandmarks);
	Temp     = 0.5*(Temp+Temp.t());
	Ckk.rows(vLandmarks) = Temp; // symmetrize
	nyKKW    = pinv(Temp);
	Temp.reset();
	nyKKC    = Ckk;	
	Ckk.reset();
	mat eC, vA;

	eig_ny(nyKKC, nyKKW, eC, vA);
	S = sqrt(vA);

	uvec indices = sort_index(diagvec(S),"descend");
	U       = eC.cols(indices);
}
#endif
