/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * kernel-matrix.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "kernel-matrix.h"
#include "elm_kernel_xy.hpp"

KernelMatrix::KernelMatrix(int iKernelType)
{
	m_iKernelType = iKernelType;
	m_status      = false;
}

KernelMatrix::~KernelMatrix()
{
}

bool KernelMatrix::Initialize(const mat &DataX,const mat &DataY, const mat &Parameters) 
{
	m_status = GenerateKernelMatrix(DataX, DataY, Parameters);
	return m_status;
}

const mat& KernelMatrix::GetKernelMatrix(void) const
{
	return m_KernelMatrix;
}

int KernelMatrix::GetKernelType(void) const
{
	return m_iKernelType;
}

bool KernelMatrix::GetStatus(void) const
{
	return m_status;
}

void KernelMatrix::size(unsigned int &rows, unsigned int &cols) const
{
	rows = m_KernelMatrix.n_rows;
	cols = m_KernelMatrix.n_cols;
}

mat  KernelMatrix::Evaluate(const uvec &X, const uvec &Y) const
{
	return m_KernelMatrix(X,Y);
}

bool KernelMatrix::GenerateKernelMatrix(const mat &DataX, const mat &DataY, const mat &Parameters)
{
	switch(m_iKernelType)
	{
		case 0:
			m_KernelMatrix = DataX*DataY.t();
			break;
		case 1:
			m_KernelMatrix = CalculateELMKernelXY(DataX,DataY);
			break;
		default:
			// the first column contains the index of the points 
			// we take this to select the correct sub-set of the kernel matrix
			// but else DataX should equal DataY
/*			std::cout << DataX.col(0) << std::endl;
			std::cout << " -- " << std::endl;
			std::cout << DataY.col(0) << std::endl;
			std::cout << Parameters.n_cols << std::endl;
			std::cout << Parameters.n_rows << std::endl;*/
			m_KernelMatrix = Parameters.rows(conv_to<uvec>::from(DataX.col(0))); //! second argument is expected to contain the respective kernel matrix
//			std::cout <<			m_KernelMatrix << std::endl; 
			m_KernelMatrix = m_KernelMatrix.cols(1,m_KernelMatrix.n_cols-1); // skip first column
			m_KernelMatrix = m_KernelMatrix.cols(conv_to<uvec>::from(DataY.col(0)));
			break;
			// nothing to do data is the kernel matrix already

	}
	return true;
}
