/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * nystroem-kernel-matrix.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "nystroem-kernel-matrix.h"

NystroemKernelMatrix::NystroemKernelMatrix():KernelMatrix(2)
{
}

NystroemKernelMatrix::~NystroemKernelMatrix()
{

}

bool NystroemKernelMatrix::Initialize(const mat &DataX, const mat &DataY, const mat &Parameters) 
{
	m_status = GenerateKernelMatrix(DataX, DataY, Parameters);
	return m_status;
}

mat NystroemKernelMatrix::Evaluate(const uvec &X, const uvec &Y) const
{
	return m_C.rows(m_vXSelector(X)) * m_Wpinv * (m_C.rows(m_vYSelector(Y))).t();
}

bool NystroemKernelMatrix::GenerateKernelMatrix(const mat &DataX, const mat &DataY, const mat &Parameters)
{
	// the kernel matrix is not generated per default to avoid large memory consumptions
	// the first column contains the index of the points
	// the second column indicates whether the point is a landmark or not
	// we take this to select the correct sub-set of the kernel matrix
	// but else DataX should equal DataY		
	m_vXSelector = conv_to<uvec>::from(DataX.col(0));
	m_vYSelector = conv_to<uvec>::from(DataY.col(0));
//	std::cout << Parameters << std::endl;
	m_C          = Parameters.cols(2,Parameters.n_cols-1);                      // approximated kernel part C - skip index and indicator column
	m_Landmarks  = conv_to<uvec>::from(Parameters.col(1));
	m_Landmarks  = find(m_Landmarks!=0);
	mat W        = m_C.rows(m_Landmarks);
//	std::cout << W << std::endl;
	W            = 0.5*(W+W.t());
	m_Wpinv      = pinv(W);
	return true;
}

void NystroemKernelMatrix::size(unsigned int &rows, unsigned int &cols) const
{
	rows = m_vXSelector.n_elem;
	cols = m_vYSelector.n_elem;
}

mat NystroemKernelMatrix::GetParameter (int iParameter) const
{
	switch(iParameter)
	{
		case 1: 
				return m_C.rows(m_vXSelector);
		case 2: 
				return m_Wpinv;
		case 3: 
				return conv_to<mat>::from(m_Landmarks);
		case 4:
				return conv_to<mat>::from(m_vXSelector);
		case 5:
				return conv_to<mat>::from(m_vYSelector);		
	}
}
