/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * pcvmmodel.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "pcvmmodel.h"
pcvm_model::pcvm_model(arma::vec Weights, double dBias, int iKernelType)
{
	m_SupportIndices = find(Weights);	
	m_Weights = Weights(m_SupportIndices);
	m_dBias   = dBias;
	m_iKernelType = iKernelType;		
}

vec pcvm_model::CalculatePrediction(const IKernelMatrixInterface &oKernelInstance,arma::uvec ReferenceIndices) const
{
	arma::uvec vModelSupportIndices = GetSupportIndices();
	arma::uvec vSelector = zeros<uvec>(vModelSupportIndices.n_elem);
	for(unsigned int i=0; i<vModelSupportIndices.n_elem;++i)
	{
		vSelector(i) = as_scalar(find(ReferenceIndices==vModelSupportIndices(i)));
	}
	unsigned int rows=0, cols = 0;
	oKernelInstance.size(rows, cols);
	uvec vDummy = linspace<uvec>(0, cols-1, cols);
	arma::mat KernelMapping = oKernelInstance.Evaluate(vSelector,vDummy);
	return Predict(KernelMapping);
}

vec pcvm_model::CalculatePrediction(const mat &RawKernelMatrix) const
{
	return Predict(RawKernelMatrix);
}


arma::vec pcvm_model::Predict(arma::mat Data) const
{
	return Data.t() * m_Weights + m_dBias;
}

arma::vec pcvm_model::GetWeights(void) const
{
	return m_Weights;
}

double pcvm_model::GetBias(void) const
{
	return m_dBias;
}

int pcvm_model::GetKernelType(void) const
{
	return m_iKernelType;
}

arma::uvec pcvm_model::GetSupportIndices(void) const
{
	return m_SupportIndices;
}

