/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * pcvmmulticlassmodel_each.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "pcvmmulticlassmodel_each.h"
#include "nystroem-kernel-matrix_filtered.h"
#include "math_stat_helper.hpp"
#include "kernel-matrix.h"
#include <vector>
#include <boost/bind.hpp>
#include <typeinfo>
using namespace std;

mat pcvm_multiclass_model_each::CalculateModelPredictions(const IKernelMatrixInterface &oKernelInstance) const
{
  arma::vec vUniqueLabelFullModel     = sm_vUniqueLabel;
  std::vector<pcvm_model> vPCVMModels = sm_Model;
  std::vector<vec> vPredictions;
  arma::uvec ReferenceIndices         = sm_vReferenceIndices;
  unsigned int iRows = 0;
  unsigned int iCols = 0;
  oKernelInstance.size(iRows,iCols);
  unsigned int N = iCols;
  vPredictions.resize(vUniqueLabelFullModel.size());

  unsigned int C = sm_vUniqueLabel.n_elem;
  int iCounter = 0;
  mat mPredictionMatrix = zeros<mat>(iCols,vPCVMModels.size());
  mat mVoteMatrix       = zeros<mat>(iCols,vPCVMModels.size());
  mat mVotingOptions    = zeros<mat>(vPCVMModels.size(),2);
  mat mPredictions      = zeros<mat>(N,vUniqueLabelFullModel.size());

  for(unsigned int k=0; k<C;k++)
  {
		for(unsigned int l=k+1; l<C; l++)
		{			
			bool bResult;
			const pcvm_model &oCurrentModel = vPCVMModels[iCounter];
			vec vCurrentPredictions;

			uvec vCurrentLabelIndices = conv_to< arma::uvec >::from(m_vvClassIndices[iCounter]);
			
			arma::uvec vModelSupportIndices = oCurrentModel.GetSupportIndices();
//			std::cout << vModelSupportIndices << std::endl;
			arma::uvec vSelector = zeros<uvec>(vModelSupportIndices.n_elem);
			for(unsigned int i=0; i<vModelSupportIndices.n_elem;++i)
			{
				vSelector(i) = as_scalar(find(ReferenceIndices==vCurrentLabelIndices(vModelSupportIndices(i))));
			}
/*			std::cout << "Selector: " << std::endl;
			std::cout << vSelector << std::endl;
	*/		unsigned int NPoints, iDummy; 
			oKernelInstance.size(iDummy,NPoints);
			const mat RawKernelMatrix = oKernelInstance.Evaluate(vSelector,linspace<uvec>(0,NPoints-1,NPoints));
/*			std::cout << "KernelMatrix:" << std::endl;			
			std::cout << RawKernelMatrix << std::endl;
	*/		vCurrentPredictions = oCurrentModel.CalculatePrediction(RawKernelMatrix);

			mPredictionMatrix.col(iCounter) = vCurrentPredictions;
			vec vCurrentLabels;
			vCurrentLabels << vUniqueLabelFullModel(k) << vUniqueLabelFullModel(l);
			mVotingOptions.row(iCounter)    = vCurrentLabels.t();
			iCounter++;
		}				
	}
//	std::cout << mPredictionMatrix << std::endl;
//			std::cout << "start voting ..."<<std::endl;
	umat mIndicator = 1-mPredictionMatrix<0;
	for(unsigned int k=0;k<iCols;k++)
	{
		for(unsigned int l=0; l<iCounter;l++)
		{		
			mVoteMatrix(k,l) = mVotingOptions(l,mIndicator(k,l));
		}
	

		for(unsigned int l=0; l<C; l++)
		{
			uvec vCurrentModels    = find(mVotingOptions.col(0)==vUniqueLabelFullModel(l));
			uvec vCurrentModels2nd = find(mVotingOptions.col(1)==vUniqueLabelFullModel(l));
			if(vCurrentModels.n_elem>0)
			{	
				if(vCurrentModels2nd.n_elem>0)
				{
					vCurrentModels    = join_cols(vCurrentModels,vCurrentModels2nd);
				}
			}
			else
			{
				vCurrentModels = vCurrentModels2nd;
			}
			vec  vSigns       = ones(sum(mVotingOptions.col(0)==vUniqueLabelFullModel(l))); // vector of ones for c vs others models
			int dSumCol2      = sum(mVotingOptions.col(1)==vUniqueLabelFullModel(l));
			if(vSigns.n_elem>0)
			{
				if(dSumCol2>0)
					vSigns      	  = join_cols(vSigns,-ones(dSumCol2)); // vector of ones for others vs c models
			}
			else
			{
					vSigns      	  = -ones(dSumCol2); // vector of ones for others vs c models
			}
			uvec vCurrentRow; 
			vCurrentRow << k;
//			std::cout << vSigns << std::endl;
//			mPredictions(k,l) = 1.0/(double)(C-1)*sum(vectorise(normalCDF(-vSigns.t() % mPredictionMatrix.submat(vCurrentRow,vCurrentModels))));
			mPredictions(k,l) = sum(vectorise(normalCDF(-vSigns.t() % mPredictionMatrix.submat(vCurrentRow,vCurrentModels))));
		}
	
	}
	mPredictions.each_col() /= sum(mPredictions.t()).t();
//	std::cout <<	mPredictions  << std::endl; 
	uvec vCrispPrediction = zeros<uvec>(iCols);
/*
	if(vUniqueLabelFullModel.n_elem >2) // basically only one model on stack
	{
		umat ModeHist = hist(mVoteMatrix,vUniqueLabelFullModel,1);
		uvec MaxValues= max(ModeHist.t()).t();
		//mPredictions = conv_to<mat>::from(ModeHist);
		for(unsigned int k=0;k<iCols;k++)
		{
			vCrispPrediction(k) = as_scalar(find(ModeHist.row(k)==MaxValues(k),1));
		}
	}
	else
	{
		for(unsigned int k=0;k<iCols;k++)
		{
			vCrispPrediction(k) = as_scalar(find(vUniqueLabelFullModel==mVoteMatrix(k,0)));
		}		    
	}
*/

	for(unsigned int k=0;k<iCols;k++)
	{
		uword  idx;
		double dMaxValue = mPredictions.row(k).max(idx);
	//	std::cout <<  vUniqueLabelFullModel(idx) << std::endl;
		vCrispPrediction(k) = idx;
	}		    

//	std::cout << " - " << std::endl;
  mat PredictionResults = zeros<mat>(N,vUniqueLabelFullModel.size()+1);
  PredictionResults.cols(0,vUniqueLabelFullModel.size()-1) = mPredictions;
  PredictionResults.col(vUniqueLabelFullModel.size())      = conv_to<vec>::from(vCrispPrediction);
  return PredictionResults;
}

pcvm_model pcvm_multiclass_model_each::CalculateClassificationModel (int i,IPCVMTrainInterface &oPCVMTrain,const IKernelMatrixInterface &oKernel, const vec &Label) const  
{  
	vec vNewLabelVector  = ones<vec>(Label.size(),1);
	uvec vValid = find(Label == i);
	vNewLabelVector(vValid) = -vNewLabelVector(vValid);
	oPCVMTrain.Apply(oKernel,vNewLabelVector);
	std::cout << "*";
	std::cout << std::flush;
	return oPCVMTrain.GetModel();
}


void pcvm_multiclass_model_each::GenerateMultiClassModels(IPCVMTrainInterface &oPCVMTrain,const IKernelMatrixInterface &oKernel, const mat &Data, const vec &Label)
{
	pcvm_multiclass_model_each &sFullModel = *this;
	arma::vec vUniqueLabel = unique(Label);
	std::vector<pcvm_model> vPCVMModels;
	std::vector<uvec>  vvClassIndices;
	std::cout << " - " << std::endl;

	int iKernel = oKernel.GetKernelType();
	bool bNystroem = false;
	IKernelMatrixInterface *oTwoClassKernel = NULL;
	if(typeid(oKernel)==typeid(NystroemKernelMatrix)) // do we actually have a Nystroem approximated kernel ?
	{
		oTwoClassKernel = new NystroemKernelMatrixFiltered();
		bNystroem = true;
	}
	else
	{
		oTwoClassKernel	 = oKernel.clone();
	}
	unsigned int C = vUniqueLabel.n_elem;
	for(unsigned int k=0; k<C;k++)
	{
		for(unsigned int l=k+1; l<C; l++)
		{
			uvec vCurrentLabelIndices = find(Label == vUniqueLabel(k) || Label == vUniqueLabel(l));
			vec vCurrentLabeling      = Label(vCurrentLabelIndices);			
			arma::vec vCurrentUniqueLabel = unique(vCurrentLabeling);
			bool bResult;
			
			if(iKernel != 2)
			{					
				bResult = oTwoClassKernel->Initialize((Data.rows(vCurrentLabelIndices)),(Data.rows(vCurrentLabelIndices)));
			}
			else
			{		
				if(bNystroem)
				{
					bResult = (dynamic_cast<NystroemKernelMatrixFiltered*>(oTwoClassKernel))->Initialize(vCurrentLabelIndices,vCurrentLabelIndices,*dynamic_cast<const NystroemKernelMatrix*>(&oKernel));
				}
				else
					bResult = oTwoClassKernel->Initialize(conv_to<mat>::from(vCurrentLabelIndices),Data.rows(vCurrentLabelIndices),Data);
			}							
			pcvm_model oCurrentPCVMModel = CalculateClassificationModel(vUniqueLabel(k),oPCVMTrain,*oTwoClassKernel,vCurrentLabeling);
			vPCVMModels.push_back(oCurrentPCVMModel);
			vvClassIndices.push_back(vCurrentLabelIndices);
		}
	}
	delete oTwoClassKernel;
	std::cout << " - " << std::endl;
	sFullModel.sm_vUniqueLabel = vUniqueLabel;
	sFullModel.sm_Model        = vPCVMModels;

	// identify the reference data points we have to keep by
	// analyzing the support indices of the various models
	arma::uvec vUniqueSupportIndices; 
	for(unsigned int k=0; k<vPCVMModels.size(); ++k)
	{
		if(k==0)
			vUniqueSupportIndices =	vvClassIndices[k](vPCVMModels[k].GetSupportIndices ());
		else
			vUniqueSupportIndices = unique(join_vert(vUniqueSupportIndices,vvClassIndices[k](vPCVMModels[k].GetSupportIndices ())));
	}
	sFullModel.sm_vReferenceIndices = vUniqueSupportIndices; 

	m_vvClassIndices.clear();
	typedef std::vector<unsigned int> uintvec;
	for(unsigned int k=0; k<vPCVMModels.size(); ++k)
	{
		m_vvClassIndices.push_back(conv_to< uintvec >::from(vvClassIndices[k]));
	}
			
	// reference data are only needed for vectorial data
	// so we keep sFullModel.sm_vvReferenceData
	if(iKernel != 2)
	{						
		int NPoints = vUniqueSupportIndices.n_elem;
		typedef std::vector<double> stdvec;
		std::vector< stdvec > vvData;
		vvData.resize(NPoints);
		for(int k=0; k<NPoints; k++)
		{
			arma::vec vTemp    = vectorise(Data.row(vUniqueSupportIndices(k)));
			vvData[k] = arma::conv_to< stdvec >::from(vTemp);
		}
		sFullModel.sm_vvReferenceData = vvData;
	}
	else
	{
		sFullModel.sm_vvReferenceData.clear();
	}
}

