/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * pcvmmulticlassmodel_ecoc.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "pcvmmulticlassmodel_ecoc.h"
#include "nystroem-kernel-matrix_filtered.h"
#include "math_stat_helper.hpp"
#include "kernel-matrix.h"
#include <vector>
#include <boost/bind.hpp>
#include <typeinfo>
using namespace std;

mat pcvm_multiclass_model_ecoc::CalculateModelPredictions(const IKernelMatrixInterface &oKernelInstance) const
{
  arma::vec vUniqueLabelFullModel     = sm_vUniqueLabel;
  std::vector<pcvm_model> vPCVMModels = sm_Model;
  std::vector<vec> vPredictions;
  arma::uvec ReferenceIndices         = sm_vReferenceIndices;
  unsigned int iRows = 0;
  unsigned int iCols = 0;
  oKernelInstance.size(iRows,iCols);
  unsigned int N = iCols;
  vPredictions.resize(vUniqueLabelFullModel.size());

  unsigned int C = sm_vUniqueLabel.n_elem;
  unsigned int L = vPCVMModels.size();
  int iCounter = 0;
  mat mPredictionMatrix = zeros<mat>(iCols,vPCVMModels.size());
  for(unsigned int k=0; k<L;k++)
  {
		bool bResult;
		const pcvm_model &oCurrentModel = vPCVMModels[iCounter];
		uvec vCurrentLabelIndices = conv_to< arma::uvec >::from(m_vvClassIndices[iCounter]);
		
		arma::uvec vModelSupportIndices = oCurrentModel.GetSupportIndices();
		arma::uvec vSelector = zeros<uvec>(vModelSupportIndices.n_elem);
		for(unsigned int i=0; i<vModelSupportIndices.n_elem;++i)
		{
			vSelector(i) = as_scalar(find(ReferenceIndices==vCurrentLabelIndices(vModelSupportIndices(i))));
		}
		
		unsigned int NPoints, iDummy; 
		oKernelInstance.size(iDummy,NPoints);
		const mat RawKernelMatrix = oKernelInstance.Evaluate(vSelector,linspace<uvec>(0,NPoints-1,NPoints));
		vec vCurrentPredictions = oCurrentModel.CalculatePrediction(RawKernelMatrix);
		mPredictionMatrix.col(iCounter) = vCurrentPredictions; // predictions for all points
		iCounter++;
 }

uvec	vCrispPrediction = zeros<uvec>(N);
for(unsigned int l= 0; l<N; l++)
{		
	mat Temp = sm_CodingMatrix;
	Temp.each_row()-=mPredictionMatrix.row(l); 
	vec HammingDistances = sum(abs(Temp).t()).t();
	uvec vMinDistance = find(HammingDistances == min(HammingDistances));
	vCrispPrediction(l) = vMinDistance[0]; // choose always the first
}	

//std::cout << vCrispPrediction << std::endl;
mat mPredictions = zeros<mat>(N,vUniqueLabelFullModel.size());
mat PredictionResults = zeros<mat>(N,vUniqueLabelFullModel.size()+1);
PredictionResults.cols(0,vUniqueLabelFullModel.size()-1) = mPredictions; // empty probabilities
PredictionResults.col(vUniqueLabelFullModel.size())      = conv_to<vec>::from(vCrispPrediction);
return PredictionResults;
}

pcvm_model pcvm_multiclass_model_ecoc::CalculateClassificationModel (int i,IPCVMTrainInterface &oPCVMTrain,const IKernelMatrixInterface &oKernel, const vec &Label) const  
{  
	vec vNewLabelVector  = ones<vec>(Label.size(),1);
	uvec vValid = find(Label == i);
	vNewLabelVector(vValid) = -vNewLabelVector(vValid);
	oPCVMTrain.Apply(oKernel,vNewLabelVector);
	std::cout << "*";
	std::cout << std::flush;
	return oPCVMTrain.GetModel();
}


void pcvm_multiclass_model_ecoc::GenerateMultiClassModels(IPCVMTrainInterface &oPCVMTrain,const IKernelMatrixInterface &oKernel, const mat &Data, const vec &Label)
{
	pcvm_multiclass_model_ecoc &sFullModel = *this;
	arma::vec vUniqueLabel = unique(Label);
	std::vector<pcvm_model> vPCVMModels;
	std::vector<uvec>  vvClassIndices;
	std::cout << " - " << std::endl;

	int iKernel = oKernel.GetKernelType();
	bool bNystroem = false;
	IKernelMatrixInterface *oTwoClassKernel = NULL;
	if(typeid(oKernel)==typeid(NystroemKernelMatrix)) // do we actually have a Nystroem approximated kernel ?
	{
		oTwoClassKernel = new NystroemKernelMatrixFiltered();
		bNystroem = true;
	}
	else
	{
		oTwoClassKernel	 = oKernel.clone();
	}
	unsigned int C = vUniqueLabel.n_elem;
	unsigned int L = sm_CodingMatrix.n_cols;
	//std::cout << L << std::endl;
	for(unsigned int k=0; k<L;k++)
	{
		uvec vPosIdx = find(sm_CodingMatrix.col(k)==1.0);
		uvec vNegIdx = find(sm_CodingMatrix.col(k)==-1.0);

		// identify valid (labeled) points (TODO look for more efficient solution like ismember)
		vec vCurrentLabels = zeros(Label.n_elem);  
		for(unsigned int l=0; l<vPosIdx.n_elem; l++)
		{
			uvec vRangeWithCurrentLabelPos = find(Label == vUniqueLabel(vPosIdx(l)));
			vCurrentLabels.elem(vRangeWithCurrentLabelPos) += 1.0;
		}

		for(unsigned int l=0; l<vNegIdx.n_elem; l++)
		{
			uvec vRangeWithCurrentLabelPos = find(Label == vUniqueLabel(vNegIdx(l)));
			vCurrentLabels.elem(vRangeWithCurrentLabelPos) += -1.0;
		}
 
		//those with 0 - are the classes which are skipped
		bool bResult;
		uvec vCurrentLabelIndices = find(vCurrentLabels!=0); // those are the pts which are kept in this model (labeled -1 or +1)
		if(iKernel != 2)
		{					
			bResult = oTwoClassKernel->Initialize((Data.rows(vCurrentLabelIndices)),(Data.rows(vCurrentLabelIndices)));
		}
		else
		{		
			if(bNystroem)
			{
				bResult = (dynamic_cast<NystroemKernelMatrixFiltered*>(oTwoClassKernel))->Initialize(vCurrentLabelIndices,vCurrentLabelIndices,*dynamic_cast<const NystroemKernelMatrix*>(&oKernel));
			}
			else
				bResult = oTwoClassKernel->Initialize(conv_to<mat>::from(vCurrentLabelIndices),Data.rows(vCurrentLabelIndices),Data);
		}							
		vec vCurrentLabeling = vCurrentLabels(vCurrentLabelIndices); // now this vector has only -1 and +1
		pcvm_model oCurrentPCVMModel = CalculateClassificationModel(-1,oPCVMTrain,*oTwoClassKernel,vCurrentLabeling);
		vPCVMModels.push_back(oCurrentPCVMModel);
		vvClassIndices.push_back(vCurrentLabelIndices);
	}
	delete oTwoClassKernel;
	std::cout << " - " << std::endl;
	sFullModel.sm_vUniqueLabel = vUniqueLabel;
	sFullModel.sm_Model        = vPCVMModels;

	// identify the reference data points we have to keep by
	// analyzing the support indices of the various models
	arma::uvec vUniqueSupportIndices; 
	for(unsigned int k=0; k<vPCVMModels.size(); ++k)
	{
		if(k==0)
			vUniqueSupportIndices =	vvClassIndices[k](vPCVMModels[k].GetSupportIndices ());
		else
			vUniqueSupportIndices = unique(join_vert(vUniqueSupportIndices,vvClassIndices[k](vPCVMModels[k].GetSupportIndices ())));
	}
	sFullModel.sm_vReferenceIndices = vUniqueSupportIndices; 
				
	m_vvClassIndices.clear();
	typedef std::vector<unsigned int> uintvec;
	for(unsigned int k=0; k<vPCVMModels.size(); ++k)
	{
		m_vvClassIndices.push_back(conv_to< uintvec >::from(vvClassIndices[k]));
	}

	// reference data are only needed for vectorial data
	// so we keep sFullModel.sm_vvReferenceData
	if(iKernel != 2)
	{					
		int NPoints = vUniqueSupportIndices.n_elem;
		typedef std::vector<double> stdvec;
		std::vector< stdvec > vvData;
		vvData.resize(NPoints);
		for(int k=0; k<NPoints; k++)
		{
			arma::vec vTemp    = vectorise(Data.row(vUniqueSupportIndices(k)));
			vvData[k] = arma::conv_to< stdvec >::from(vTemp);
		}
		sFullModel.sm_vvReferenceData = vvData;	
	}
	else
	{
		sFullModel.sm_vvReferenceData.clear();
	}
}


